/**
 * @file sensor_kp.c
 * @Author Kirill Ustimov <ustimov@cs.karelia.ru>
 * @date   Created on April 15, 2013
 * @brief  Consumer of sensors values using http://www.w3.org/2005/Incubator/ssn/ssnx/ssn ontology
 *
 * This is file is a part of the C KPI library project and it has the same licence. 
 *
 * @section LICENSE
 * 
 * C KPI library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * C KPI Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SmartSlog KP Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 *
 * @section DESCRIPTION
 *
 * This file is a part of the 'Hello Sensors' example.
 *
 * Copyright (C) SmartSlog Team.
 * All rights reserved.
 * Mail-list: smartslog@cs.karelia.ru
 */

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <ckpi/ckpi.h>

// ---Constants---
// Describes connection to the SS.
#define KP_SS_NAME "X"
#define KP_SS_ADDRESS "194.85.173.9"  // Public PetrSU SIB
//#define KP_SS_ADDRESS "192.168.56.3"  // Local SIB
#define KP_SS_PORT 10010

// Name of the KP.
#define KP_NAME "HelloSensor"

// Parts of the triple for the query.
#define KP_NS_URI "http://example.org/#"
#define KP_SUBJECT KP_NS_URI"KP"
#define KP_PREDICATE KP_NS_URI"says"

/****************** Static functions list **********************/
/*******************************************************/
static bool keepRunning = true;

void sigHandler(int sig) {
    keepRunning = false;
}    

/**
 * @brief Starts the KP, initializes state and cals all necessary functions.
 *
 * @return 0.
 */
int main()
{
  ss_triple_t * cur = NULL;
    ss_triple_t * rem = NULL;
  ss_bnode_t * bnodes = NULL;
  ss_info_t info;
  char value[3];
  
   signal(SIGINT, sigHandler);
  ss_init_space_info(&info, KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);
   
    // In the smart space the name of the KP must be unique, if you try to 
    // join without leave, then you received an error.
    if (ss_join(&info, KP_NAME) == -1) {
        printf("Can't join to SS\n");
        return 0;
    }
    
    printf("KP join to SS. Press ctrl+C to exit.\nPublishing values from theese sensors:\n1.TemperatureSensor\n2.LightingSensor\n3.HumiditySensor\n");
  ss_add_triple(&cur,"http://example.org/#TemperatureSensor", "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "http://purl.oclc.org/NET/ssnx/ssn#Sensor",SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  
  ss_add_triple(&cur,"http://example.org/#LightingSensor", "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "http://purl.oclc.org/NET/ssnx/ssn#Sensor",SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
ss_add_triple(&cur,"http://example.org/#HumiditySensor", "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "http://purl.oclc.org/NET/ssnx/ssn#Sensor",SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
ss_add_triple(&cur,"http://example.org/#TemperatureSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasMeasurementProperty", "Temprature",SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  
  ss_add_triple(&cur,"http://example.org/#LightingSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasMeasurementProperty", "Lighting",SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
ss_add_triple(&cur,"http://example.org/#HumiditySensor", "http://purl.oclc.org/NET/ssnx/ssn#hasMeasurementProperty", "Humidity",SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  // Create triple for removing data, this triple is not changing while 
  // Create triple for removing data, this triple is not changing while 
    // publishing process working.    
  ss_add_triple(&rem, "http://example.org/#TemperatureSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasValue", SS_RDF_SIB_ANY, 
                SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
    ss_add_triple(&rem, "http://example.org/#LightingSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasValue", SS_RDF_SIB_ANY, 
                SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
ss_add_triple(&rem, "http://example.org/#HumiditySensor", "http://purl.oclc.org/NET/ssnx/ssn#hasValue", SS_RDF_SIB_ANY, 
                SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_insert(&info, cur, bnodes);
  ss_delete_triples(cur);
  cur = NULL;
  while (keepRunning)
  {
    sprintf(value, "%d", rand() % 100);
    ss_add_triple(&cur,"http://example.org/#TemperatureSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasValue", value,SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
sprintf(value, "%d", rand() % 100);
    
    ss_add_triple(&cur,"http://example.org/#LightingSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasValue", value,SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
sprintf(value, "%d", rand() % 100);
    
    ss_add_triple(&cur,"http://example.org/#HumiditySensor", "http://purl.oclc.org/NET/ssnx/ssn#hasValue", value,SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
    ss_update(&info, cur, rem, bnodes);
    ss_delete_triples(cur);
    cur = NULL;
    sleep(6);
  }
  ss_add_triple(&rem, "http://example.org/#LightingSensor", "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "http://purl.oclc.org/NET/ssnx/ssn#Sensor",
                SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  
  ss_add_triple(&rem, "http://example.org/#TemperatureSensor", "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "http://purl.oclc.org/NET/ssnx/ssn#Sensor",
                SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
ss_add_triple(&rem,"http://example.org/#HumiditySensor", "http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "http://purl.oclc.org/NET/ssnx/ssn#Sensor",SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
ss_add_triple(&rem,"http://example.org/#TemperatureSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasMeasurementProperty", "Temprature",SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  
  ss_add_triple(&rem,"http://example.org/#LightingSensor", "http://purl.oclc.org/NET/ssnx/ssn#hasMeasurementProperty", "Lighting",SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
ss_add_triple(&rem,"http://example.org/#HumiditySensor", "http://purl.oclc.org/NET/ssnx/ssn#hasMeasurementProperty", "Humidity",SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_remove(&info, rem);
  ss_leave(&info);
  ss_delete_triples(rem);
  return 0;
}
