/**
 * @file executor_kp.c
 * @Author Ustimov K. <ustimov@cs.karelia.ru>
 * @date   Created on August 3, 2012
 * @brief  KP that allows to execute any queries.
 *
 * This is file is a part of the C KPI library project and it has the same licence. 
 *
 * @section LICENSE
 * 
 * C KPI library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * C KPI Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SmartSlog KP Library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *
 *
 * @section DESCRIPTION
 *
 * This file is a part of the 'SPARQL queries' example without subscription.
 * This KP executes SELECT, ASK or CONSTRUCT queries given by user.
 *
 * Copyright (C) SmartSlog Team.
 * All rights reserved.
 * Mail-list: smartslog@cs.karelia.ru
 */

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include <ckpi/ckpi.h>

// ---Constants---
// Describes connection to the SS.
#define KP_SS_NAME "X"
#define KP_SS_ADDRESS "194.85.173.9"  // Public PetrSU SIB
//#define KP_SS_ADDRESS "192.168.56.3"  // Local SIB
#define KP_SS_PORT 10010

// Name of the KP.
#define KP_NAME "ExecutorKP"

// Parts of the triple for the query.
#define KP_NS_URI "http://X/Hello/#"
#define KP_SUBJECT KP_NS_URI"KP"
#define KP_PREDICATE KP_NS_URI"says"

/****************** Static functions list **********************/
/*******************************************************/
        

/**
 * @brief Starts the KP, initializes state and cals all necessary functions.
 *
 * @return 0.
 */
int main()
{
    // Initialize smart space information.
    ss_info_t info;
    int choose;
    int ask_result = 0;
    int nob;
    int i;
    char input[2000] = { 0 };
    ss_triple_t * result_triple = NULL;
    ss_triple_t * current_triple = NULL;
    ss_sparql_result_t * results;
    
    ss_init_space_info(&info, KP_SS_NAME, KP_SS_ADDRESS, KP_SS_PORT);
   
    // In the smart space the name of the KP must be unique, if you try to 
    // join without leave, then you received an error.
    if (ss_join(&info, KP_NAME) == -1) {
        printf("Can't join to SS\n");
        return 0;
    }
    printf("KP join to SS.\n");
    printf("Choose query type or press 4 to exit:\n1 - SPARQL CONSTRUCT\n2 - SPARQL ASK\n3 - SPARQL SELECT\n");
    if (fgets(input, 2000, stdin) == NULL)
    {
      printf("error or eof\n");
    }
    choose = atoi(input);
    while (choose != 4)
    {
      printf("Enter query:\n");
      if (fgets(input, 2000, stdin) == NULL)
      {
	printf("error or eof\n");
      }
      switch (choose)
      {
	case 1:
	  if (ss_sparql_construct_query(&info, input, &result_triple) != 0)
	  {
	    printf("error\n");
	    break;
	  }
	  current_triple = result_triple;
	  while (current_triple != NULL)
	  {
	    printf("%s %s %s\n", current_triple->subject, current_triple->predicate, current_triple->object);
	    current_triple = current_triple->next;
	  }
	  ss_delete_triples(result_triple);
	  break;
	case 2:
	  if (ss_sparql_ask_query(&info, input, &ask_result) != 0)
	  {
	    printf("error\n");
	    break;
	  }
	  if (ask_result == 0)
	  {
	      printf("true\n");
	  }
	  else 
	  {
	      printf("false \n");
	  }
	  break;
	case 3:
	  if (ss_sparql_select_query(&info, input, &results, &nob) != 0)
	  {
	    printf("error\n");
	    break;
	  }
	  printf("number of variables = %d\n", nob);
	  while (results != NULL)
	  {
	    for(i = 0; i < nob; i++)
	    {
	      printf("name: %s type: %d value: %s\n", results->name[i], results->type[i], results->value[i]);
	    }
	    results = results->next;
	    printf("\n");
	  }
	  break;
	default:
	  printf("unknown input\n");
	  break;
      }
      printf("Choose query type or press 4 to exit:\n1 - SPARQL CONSTRUCT\n2 - SPARQL ASK\n3 - SPARQL SELECT\n");
      if (fgets(input, 2000, stdin) == NULL)
      {
	printf("error or eof\n");
      }
      choose = atoi(input);
    }
    ss_leave(&info);    

    printf("KP leave SS...\n");

    return 0;
}
